//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017-2018 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package store

import (
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"os/exec"
	"strconv"
	"strings"
	"syscall"
	"time"
)

type ffprobeFormat struct {
	NumStreams  uint              `json:"nb_streams"`
	NumPrograms uint              `json:"nb_programs"`
	FormatName  string            `json:"format_name"`
	DurationSec string            `json:"duration"`
	Size        string            `json:"size"`
	BitRate     string            `json:"bit_rate"`
	Tags        map[string]string `json:"tags"`
}

type ffprobeOutput struct {
	Format ffprobeFormat `json:"format"`
}

func (st *Store) readMetadataFromFile(filename string) (duration time.Duration, size uint64, tags map[string]string, err error) {
	ctx, cancel := context.WithTimeout(context.Background(), 60*time.Second)
	defer cancel()
	// cmd := exec.CommandContext(ctx, "ffprobe", "-hide_banner", "-print_format", "json", "-show_format", filename)
	cmd := exec.CommandContext(ctx, "ffprobe", "-print_format", "json", "-show_format", filename)
	var stdout, stderr bytes.Buffer
	cmd.Stdout = &stdout
	cmd.Stderr = &stderr
	if err = cmd.Run(); err != nil {
		if exiterr, ok := err.(*exec.ExitError); ok {
			if status, ok := exiterr.Sys().(syscall.WaitStatus); ok {
				// TODO: handle stderr?
				err = fmt.Errorf("ffprobe returned %d", status.ExitStatus())
				return
			}
		}
		err = fmt.Errorf("getting exit code of ffprobe failed: %v", err)
		return
	}

	var output ffprobeOutput
	if err = json.NewDecoder(&stdout).Decode(&output); err != nil {
		return
	}
	if duration, err = time.ParseDuration(output.Format.DurationSec + "s"); err != nil {
		err = fmt.Errorf("unable to parse duration from ffprobe output: %v", err)
		return
	}
	if size, err = strconv.ParseUint(output.Format.Size, 10, 64); err != nil {
		err = fmt.Errorf("unable to parse file-size from ffprobe output: %v", err)
		return
	}
	tags = output.Format.Tags
	return
}

func (st *Store) populateFileMetadataFields(group string, id uint64, fields map[string]interface{}) error {
	duration, size, tags, err := st.readMetadataFromFile(st.GetFilePath(group, id))
	if err != nil {
		return err
	}
	fields["duration"] = duration
	fields["size"] = size

	for tag, value := range tags {
		switch strings.ToLower(tag) {
		case "artist":
			fields["metadata__artist"] = value
		case "title":
			fields["metadata__title"] = value
		case "album":
			fields["metadata__album"] = value
		}
	}
	return nil
}
